export default `.jsPanel .jsPanel-content{background-color:var(
    --design-background-color,
    var(--panel-background-color)
  );color:var(--design-background-text-color, var(--panel-on-background-color));}`
